package com.sun.org.apache.xml.internal.security.c14n.implementations;

import java.util.ArrayList;
import java.util.List;

class SymbMap implements Cloneable
{
    int free;
    NameSpaceSymbEntry[] entries;
    String[] keys;
    
    SymbMap() {
        this.free = 23;
        this.entries = new NameSpaceSymbEntry[this.free];
        this.keys = new String[this.free];
    }
    
    void put(String obj, NameSpaceSymbEntry nameSpaceSymbEntry) {
        int index = this.index(obj);
        String s = this.keys[index];
        this.keys[index] = obj;
        this.entries[index] = nameSpaceSymbEntry;
        if ((s == null || !s.equals(obj)) && --this.free == 0) {
            this.free = this.entries.length;
            this.rehash(this.free << 2);
        }
    }
    
    List<NameSpaceSymbEntry> entrySet() {
        ArrayList<Object> list = new ArrayList();
        for (int i = 0; i < this.entries.length; ++i) {
            if (this.entries[i] != null && !"".equals(this.entries[i].uri)) {
                list.add(this.entries[i]);
            }
        }
        return list;
    }
    
    protected int index(Object o) {
        String[] keys = this.keys;
        int length = keys.length;
        int n = (o.hashCode() & Integer.MAX_VALUE) % length;
        String s = keys[n];
        if (s == null || s.equals(o)) {
            return n;
        }
        --length;
        String s2;
        do {
            n = ((n == length) ? 0 : (++n));
            s2 = keys[n];
        } while (s2 != null && !s2.equals(o));
        return n;
    }
    
    protected void rehash(int n) {
        int length = this.keys.length;
        String[] keys = this.keys;
        NameSpaceSymbEntry[] entries = this.entries;
        this.keys = new String[n];
        this.entries = new NameSpaceSymbEntry[n];
        while (length-- > 0) {
            if (keys[length] == null) {
                continue;
            }
            String s = keys[length];
            int index = this.index(s);
            this.keys[index] = s;
            this.entries[index] = entries[length];
        }
    }
    
    NameSpaceSymbEntry get(String s) {
        return this.entries[this.index(s)];
    }
    
    @Override
    protected Object clone() {
        try {
            SymbMap symbMap = (SymbMap)super.clone();
            symbMap.entries = new NameSpaceSymbEntry[this.entries.length];
            System.arraycopy(this.entries, 0, symbMap.entries, 0, this.entries.length);
            symbMap.keys = new String[this.keys.length];
            System.arraycopy(this.keys, 0, symbMap.keys, 0, this.keys.length);
            return symbMap;
        }
        catch (CloneNotSupportedException ex) {
            ex.printStackTrace();
            return null;
        }
    }
}
